use chrono::{DateTime, Utc};
use sqlx::PgPool;

use super::{function::Function, role::Role};

#[derive(Clone)]
pub struct User {
    pub id: i32,
    pub name: String,
    pub email: String,
    pub password: String,
    pub salt: String,
    pub role: Role,
    pub function: Function,
    pub area_id: i32,
    pub locked: bool,
    pub last_login: Option<DateTime<Utc>>,
    pub receive_notifications: bool,
}

impl User {
    pub async fn create(
        pool: &PgPool,
        name: String,
        email: String,
        password: String,
        salt: String,
        role: Role,
        function: Function,
        area_id: i32,
    ) -> anyhow::Result<i32> {
        let created = sqlx::query!(
            r#"
                INSERT INTO user_ (name, email, password, salt, role, function, areaId)
                VALUES ($1, $2, $3, $4, $5, $6, $7)
                RETURNING id;
            "#,
            name,
            email,
            password,
            salt,
            role as Role,
            function as Function,
            area_id
        )
        .fetch_one(pool)
        .await;

        match created {
            Ok(result) => Ok(result.id),
            Err(err) => Err(err.into()),
        }
    }

    pub async fn read_by_id(pool: &PgPool, id: i32) -> anyhow::Result<User> {
        let record = sqlx::query!(
            r#"
                SELECT id,
                    name,
                    email,
                    password,
                    salt,
                    role AS "role: Role",
                    function AS "function: Function",
                    areaId,
                    locked,
                    lastLogin,
                    receiveNotifications    
                FROM user_
                WHERE id = $1;
            "#,
            id,
        )
        .fetch_one(pool)
        .await?;

        let user = User {
                id: record.id,
                name: record.name,
                email: record.email,
                password: record.password,
                salt: record.salt,
                role: record.role,
                function: record.function,
                area_id: record.areaid,
                locked: record.locked,
                last_login: record.lastlogin,
                receive_notifications: record.receivenotifications,
        };

        Ok(user)
    }

    pub async fn read_for_login(pool: &PgPool, email: &str) -> anyhow::Result<Option<User>> {
        let record = sqlx::query!(
            r#"
                SELECT id,
                    name,
                    email,
                    password,
                    salt,
                    role AS "role: Role",
                    function AS "function: Function",
                    areaId,
                    locked,
                    lastLogin,
                    receiveNotifications    
                FROM user_
                WHERE email = $1;
            "#,
            email,
        )
        .fetch_optional(pool)
        .await?;

        let result = match record {
            Some(record) => Some(User {
                id: record.id,
                name: record.name,
                email: record.email,
                password: record.password,
                salt: record.salt,
                role: record.role,
                function: record.function,
                area_id: record.areaid,
                locked: record.locked,
                last_login: record.lastlogin,
                receive_notifications: record.receivenotifications,
            }),
            None => None
        };

        Ok(result)
    }

    pub async fn read_all(pool: &PgPool) -> Option<Vec<User>> {
        todo!()
    }

    pub async fn update(pool: &PgPool, id: i32, updated_user: User) -> Option<User> {
        todo!()
    }

    pub async fn delete(pool: &PgPool, id: i32) -> anyhow::Result<bool> {
        todo!()
    }
}
